use rocket::{
    http::{Method, Status},
    outcome::Outcome,
    response::{Redirect, Responder},
    route::Handler,
    Data, Request, Response, Route,
};

#[derive(Clone, Copy)]
pub struct Redirecter;

impl From<Redirecter> for Vec<Route> {
    fn from(redirecter: Redirecter) -> Self {
        let mut out = vec![];
        for method in [
            Method::Get,
            Method::Put,
            Method::Post,
            Method::Delete,
            Method::Options,
            Method::Head,
            Method::Trace,
            Method::Connect,
            Method::Patch,
        ] {
            let mut route = Route::ranked(50, method, "/<path..>", redirecter);
            route.name = Some("TomatenmharksiteRedirecter".into());
            out.push(route);
        }
        out
    }
}

#[rocket::async_trait]
impl Handler for Redirecter {
    // fn respond_to(self, request: &'r Request<'_>) -> Result<'o> {
    async fn handle<'r>(
        &self,
        request: &'r Request<'_>,
        _data: Data<'r>,
    ) -> Outcome<Response<'r>, Status, (Data<'r>, Status)> {
        let uri = request.uri();
        let mut path = uri.path().raw_segments();
        if let Some(which) = path.next().and_then(|v| v.percent_decode().ok()) {
            if which
                .chars()
                .all(|ch| ch.is_alphanumeric() || ch == '_' || ch == '-')
            {
                let path = path.map(|v| format!("/{v}")).collect::<String>();
                if let Some(port_or_domain) =
                    tokio::fs::read_to_string(format!("/srv/tomatenmhark-redirect/{}", which))
                        .await
                        .ok()
                        .map(|c| c.trim().parse::<u16>().map_err(|_| c))
                {
                    let redirect_target = match port_or_domain {
                        Ok(port) => {
                            let domain = request
                                .host()
                                .map(|host| host.domain().as_str())
                                .unwrap_or("tomatenmhark.org");
                            format!("http://{domain}:{port}{path}")
                        }
                        Err(domain) => {
                            let domain = domain.trim();
                            if !domain.contains("%DOMAIN%") {
                                format!("{domain}{path}")
                            } else {
                                format!(
                                    "{}/{path}",
                                    domain.replace(
                                        "%DOMAIN%",
                                        request
                                            .host()
                                            .map(|host| host.domain().as_str())
                                            .unwrap_or("tomatenmhark.org")
                                    )
                                )
                            }
                        }
                    };
                    if let Ok(redirect) = Redirect::temporary(redirect_target).respond_to(request) {
                        Outcome::Success(redirect)
                    } else {
                        Outcome::error(Status::InternalServerError)
                    }
                } else {
                    Outcome::error(Status::NotFound)
                }
            } else {
                Outcome::error(Status::NotFound)
            }
        } else {
            Outcome::error(Status::NotFound)
        }
    }
}
